import { Cookies } from './cookie.js';
type RespondBody = string | object | Uint8Array | ArrayBuffer;
interface RespondInit {
    /**
     * Outgoing Headers
     */
    headers?: HeadersInit;
    /**
     * Response status code
     * - 200 Ok (default)
     * - 300 Redirect
     * - 400 Client Error
     * - 500 Server Error
     */
    status?: number;
}
interface AcceptInit {
    /**
     * Accept: `text/html`
     * - content: 'text/html'
     * */
    content?: string;
    /**
     * Accept: `text/html`
     * - type: 'text'
     * */
    type?: string;
    /**
     * Accept: `text/html`
     * - sub: 'html'
     * */
    sub?: string;
    /**
     * Accept: `application/<xhtml>+<xml>`
     * - suffix: `xhtml`
     * - sub: `xml`
     * */
    suffix?: string;
    /**
     * Accept: `q=<weight>`
     * - weight: '0.8'
     * */
    weight?: string;
}
interface JsonInit extends RespondInit {
    replacer?: (this: any, key: string, value: any) => any;
    space?: string | number;
}
export declare class Context {
    #private;
    readonly method: string;
    readonly url: URL;
    readonly headers: Headers;
    header: Headers;
    constructor(request: Request);
    get request(): Request;
    get response(): Response;
    get pathname(): string;
    /**
     * Search params from uri
     * */
    get query(): URLSearchParams;
    /**
     * Uri params from `ctx.urlPattern.pathname.groups`
     * @example
     * '/search/:query' => {query: 'text'}
     *
     * @see {@link https://developer.mozilla.org/en-US/docs/Web/API/URL_Pattern_API URLPattern API}
     * */
    get params(): {
        [key: string]: string;
    };
    /**
     * The urlPattern object is only available on the route
     *
     * {@link https://developer.mozilla.org/en-US/docs/Web/API/URL_Pattern_API URLPattern API}
     * */
    get urlPattern(): URLPatternResult;
    /**
     * Return cookie object from Request
     * ```
     * {foo: 'bar'}
     * ```
     */
    get cookie(): Record<string, string>;
    /**
     * Cookies for Response
     * @example
     * ctx.cookies.set({name: 'foo', value: 'bar', httpOnly: true, maxAge: 3600})
     *
     * ctx.cookies.delete({name: 'foo'})
     * */
    get cookies(): Cookies;
    /**
     * @description Return current status code
     * */
    get status(): number;
    /**
     * Return response status
     * */
    get responded(): boolean;
    /**
     * Get respond data
     */
    static Response(ctx: Context): Response;
    static SetUrlPatternResult(ctx: Context, urlPattern: URLPatternResult): void;
    /**
     * Write response to client
     *
     * @example
     * ctx.respondWith(new Response('data'))
     * */
    respondWith(response: Response): void;
    /**
     * Send data
     */
    respond(body?: RespondBody, init?: RespondInit): void;
    /**
     * Respond application/json
     */
    json(data?: any, init?: JsonInit): void;
    /**
     * Respond text/plain
     */
    text(data?: string, init?: RespondInit): void;
    /**
     * Redirect Status Code
     * - 301 Moved Permanently
     * - 302 Found, the temporary redirect
     * - 303 See Other (default)
     * - 304 Not Modified
     * - 307 Temporary Redirect
     * - 308 Permanent Redirect
     */
    redirect(uri: string | URL, status?: number): void;
    accept(init: AcceptInit): true | void;
    /**
     * Set Content-Type header
     */
    type(mimeType: string): this;
    /**
     * Set `ETag` header
     *
     * return 304 status if `(etag == if-none-match)`
     */
    etag(etag: string): void;
    /**
     * - Set `Last-Modified` header
     *
     * return 304 status if `(last-modified == if-modified-since)`
     */
    lastModified(date: Date): void;
}
export {};
