import { Readable } from 'node:stream';
import { pipeline } from 'node:stream/promises';
const isStream = (response) => {
    return response.headers.has('content-encoding') ||
        response.headers.has('transfer-encoding') ||
        response.headers.has('content-length') ||
        !/^(application\/json\b|text\/(?!event-stream\b))/i.test(response.headers.get('content-type'));
};
const headersToObject = (headers) => {
    if (!headers)
        return {};
    let setCookie;
    let header = {};
    for (const [key, val] of headers) {
        if (key === 'set-cookie') {
            if (!setCookie)
                header[key] = setCookie = [];
            setCookie.push(val);
            continue;
        }
        header[key] = val;
    }
    return header;
};
export const requestToWeb = (req) => {
    const url = new URL(req.url, `http://${req.headers['host']}`);
    const init = {
        method: req.method,
        headers: req.headers,
    };
    if (!(init.method === 'GET' || init.method === 'HEAD')) {
        init.body = Readable.toWeb(req);
        init.duplex = 'half';
    }
    return new Request(url, init);
};
export const h2streamToWeb = (stream, headersRaw) => {
    const url = new URL(headersRaw[':path'], `${headersRaw[':scheme']}://${headersRaw[':authority'] || headersRaw['host']}`);
    const init = {
        method: headersRaw[':method'],
        headers: headersRaw,
    };
    if (!(init.method === 'GET' || init.method === 'HEAD')) {
        init.body = Readable.toWeb(Readable.from(stream));
        init.duplex = 'half';
    }
    return new Request(url, init);
};
export const respondWith = async (res, response) => {
    if (!response)
        throw new Error('Respond not provided');
    if (response.type === 'error')
        return res.destroy();
    if (response.body) {
        try {
            if (isStream(response)) {
                res.writeHead(response.status, headersToObject(response.headers));
                await pipeline(Readable.fromWeb(response.body), res);
                return;
            }
            const arrayBuffer = await response.arrayBuffer();
            response.headers.set('content-length', `${arrayBuffer.byteLength}`);
            res.writeHead(response.status, headersToObject(response.headers));
            // res.end(Buffer.from(arrayBuffer))
            res.end(new Uint8Array(arrayBuffer));
        }
        catch (e) {
            console.error(e);
            const err = e instanceof Error ? e : new Error('unknown error', { cause: e });
            res.destroy(err);
        }
    }
    else {
        res.writeHead(response.status, headersToObject(response.headers));
        res.end();
    }
};
export const h2respondWith = async (stream, response) => {
    if (!response)
        throw new Error('Respond not provided');
    if (response.type === 'error')
        return stream.destroy();
    if (response.body) {
        try {
            if (isStream(response)) {
                stream.respond({
                    ':status': response.status,
                    ...headersToObject(response.headers),
                });
                await pipeline(Readable.fromWeb(response.body), stream);
                return;
            }
            const arrayBuffer = await response.arrayBuffer();
            response.headers.set('content-length', `${arrayBuffer.byteLength}`);
            stream.respond({
                ':status': response.status,
                ...headersToObject(response.headers),
            });
            stream.end(Buffer.from(arrayBuffer));
        }
        catch (e) {
            console.error(e);
            const err = e instanceof Error ? e : new Error('unknown error', { cause: e });
            stream.destroy(err);
        }
    }
    else {
        stream.respond({
            ':status': response.status,
            ...headersToObject(response.headers),
        });
        stream.end();
    }
};
