import 'urlpattern-polyfill';
import { Context } from '../context.js';
import { HttpMethod } from '../httpMethod.js';
export type RouteHandle = (ctx: Context) => any;
export type MiddlewareHandle = (ctx: Context, done?: () => Promise<void>) => void;
interface RoutesHandler {
    pattern: URLPattern;
    handle: RouteHandle;
}
interface RouterOptions {
    prefix?: string;
}
export declare class Router {
    #private;
    readonly init: RouterOptions;
    constructor(init?: RouterOptions);
    static Exec(router: Router, ctx: Context): Promise<Response>;
    setRouter(router: Router): void;
    /**
     * @example
     * router.add('GET', '/posts', ctx => {
     *   ctx.respond(`Posts`)
     * })
     *
     * @see {@link https://developer.mozilla.org/en-US/docs/Web/API/URL_Pattern_API URLPattern Syntax}
     * */
    add(method: HttpMethod, path: string, handle: RouteHandle): void;
    /**
     * @example
     * router.add('GET', {pathname: '/posts'}, ctx => {
     *   ctx.respond(`Posts`)
     * })
     *
     * @see {@link https://developer.mozilla.org/en-US/docs/Web/API/URL_Pattern_API URLPattern Syntax}
     * */
    add(method: HttpMethod, pattern: URLPatternInit | string, handle: RouteHandle): void;
    /**
     * Add Middleware
     * ```ts
     *  app.router.use(async (ctx, done) => {
     *    let ts = performance.now()
     *    await done()
     *    ctx.header.set('x-time', (performance.now() - ts).toFixed(4))
     *  })
     * ```
     * */
    use(handle: MiddlewareHandle): this;
    /**
     * Add GET route
     *
     * @example
     * router.get('/posts', ctx => {
     *   ctx.respond(`Posts`)
     * })
     *
     * router.get('/posts/:id(\\d+)', ctx => {
     *   ctx.json({id: ctx.params['id']})
     * })
     *
     * @see {@link https://developer.mozilla.org/en-US/docs/Web/API/URL_Pattern_API URLPattern Syntax}
     * */
    get(path: string, handle: RouteHandle): void;
    get(pattern: URLPatternInit, handle: RouteHandle): void;
    /**
     * Add POST route
     *
     * @example
     * router.post('/posts/:id', async ctx => {
     *   const body = await ctx.body.json()
     *   ctx.json({
     *     id: ctx.params['id']
     *     data: body
     *   })
     * })
     *
     * @see {@link https://developer.mozilla.org/en-US/docs/Web/API/URL_Pattern_API URLPattern Syntax}
     * */
    post(path: string, handle: RouteHandle): void;
    post(pattern: URLPatternInit, handle: RouteHandle): void;
    put(path: string, handle: RouteHandle): void;
    put(pattern: URLPatternInit, handle: RouteHandle): void;
    patch(path: string, handle: RouteHandle): void;
    patch(pattern: URLPatternInit, handle: RouteHandle): void;
    delete(path: string, handle: RouteHandle): void;
    delete(pattern: URLPatternInit, handle: RouteHandle): void;
    options(path: string, handle: RouteHandle): void;
    options(pattern: URLPatternInit, handle: RouteHandle): void;
    head(path: string, handle: RouteHandle): void;
    head(pattern: URLPatternInit, handle: RouteHandle): void;
    match(method: HttpMethod, uri: URL): Generator<RoutesHandler, void, unknown>;
    middleware(): Iterable<MiddlewareHandle>;
}
export {};
