import 'urlpattern-polyfill';
import { Context } from '../context.js';
import { pathnameNormalize } from '../lib/utils.js';
export class Router {
    init;
    #routes = new Map();
    #middleware = [];
    #externalRouter = [];
    constructor(init = {}) {
        this.init = init;
    }
    static async Exec(router, ctx) {
        // handle middleware
        let promiseStack = [];
        for (const handle of router.middleware()) {
            handle(ctx, () => new Promise(resolve => promiseStack.push(resolve)));
        }
        // handle routes
        for (const match of router.match(ctx.method, ctx.url)) {
            Context.SetUrlPatternResult(ctx, match.pattern.exec(ctx.url));
            await match.handle(ctx);
            if (ctx.responded)
                break;
        }
        // middleware resolved
        // for (let resolve of promiseStack.reverse()) resolve()
        while (promiseStack.length > 0) {
            await promiseStack.pop()();
        }
        return Context.Response(ctx);
    }
    setRouter(router) {
        if (this === router)
            throw new Error('Router cannot add itself');
        if (this.#externalRouter.includes(router))
            throw new Error('Router already added');
        this.#externalRouter.push(router);
    }
    add(method, pattern, handle) {
        if (!this.#routes.has(method))
            this.#routes.set(method, []);
        const routes = this.#routes.get(method);
        if (typeof pattern == 'string')
            pattern = { pathname: pattern };
        if (this.init.prefix)
            pattern.pathname = pathnameNormalize(this.init.prefix, pattern.pathname);
        const urlPattern = new URLPattern(pattern);
        routes.push({ pattern: urlPattern, handle });
        return this;
    }
    /**
     * Add Middleware
     * ```ts
     *  app.router.use(async (ctx, done) => {
     *    let ts = performance.now()
     *    await done()
     *    ctx.header.set('x-time', (performance.now() - ts).toFixed(4))
     *  })
     * ```
     * */
    use(handle) {
        this.#middleware.push(handle);
        return this;
    }
    get(pattern, handle) {
        this.add('GET', pattern, handle);
        return this;
    }
    post(pattern, handle) {
        this.add('POST', pattern, handle);
        return this;
    }
    put(pattern, handle) {
        this.add('PUT', pattern, handle);
        return this;
    }
    patch(pattern, handle) {
        this.add('PATCH', pattern, handle);
        return this;
    }
    delete(pattern, handle) {
        this.add('DELETE', pattern, handle);
        return this;
    }
    options(pattern, handle) {
        this.add('OPTIONS', pattern, handle);
        return this;
    }
    head(pattern, handle) {
        this.add('HEAD', pattern, handle);
        return this;
    }
    *match(method, uri) {
        if (this.#routes.has(method)) {
            for (const route of this.#routes.get(method)) {
                if (route.pattern.test(uri)) {
                    yield route;
                }
            }
        }
        // External
        for (const router of this.#externalRouter) {
            if (router.#routes.has(method)) {
                for (const route of router.#routes.get(method)) {
                    if (route.pattern.test(uri)) {
                        yield route;
                    }
                }
            }
        }
    }
    *middleware() {
        for (const middleware of this.#middleware) {
            yield middleware;
        }
        for (const router of this.#externalRouter) {
            yield* router.middleware();
        }
    }
}
